<?php

/**
 * This class is used to present data in an audit log in a way that is useful to a user.
 * Mainly this will deal with assocaited ids and lookup the actual value of the record instead
 * of just the id being displayed. When presenting an audit delta value, the presenter will
 * attempt to call out to the plugin that the audit belongs so that the value can be found
 * within that plugin. If no presenter is found in the plugin then the value is returned without
 * modification.
 */
class AuditLogPresenter {

/**
 * The item name identifies the item that is changing in the audit
 *
 * @param array $audit he audit data.
 * @return string Humanised item name. Defaults to "ModelName #id"
 */
	public function presentItemName($audit) {
		list($plugin, $modelAlias) = pluginSplit($audit['model']);
		$default = Inflector::humanize($modelAlias) . ' #' . $audit['entity_id'];
		return self::_callPluginPresenter('customItemName', $audit, null, $default);
	}

/**
 * The property name is the name of the database field that changed. This method turns the field name
 * into something readable.
 *
 * @param array $audit      The audit data.
 * @param array $auditDelta The aduit delta array.
 * @return string Humanised property name.
 */
	public function presentPropertyName($audit, $auditDelta) {
		if (empty($auditDelta['property_name'])) {
			return '';
		}

		return Inflector::humanize($auditDelta['property_name']);
	}

/**
 * Present an old value of an audit delta.
 *
 * @param array $audit      The audit data.
 * @param array $auditDelta The aduit delta array.
 * @return string The value to present.
 */
	public function presentOldValue($audit, $auditDelta) {
		return self::presentValue($audit, $auditDelta, 'old');
	}

/**
 * Present an new value of an audit delta.
 *
 * @param array $audit      The audit data.
 * @param array $auditDelta The aduit delta array.
 * @return string The value to present.
 */
	public function presentNewValue($audit, $auditDelta) {
		return self::presentValue($audit, $auditDelta, 'new');
	}

/**
 * Present a audit delta value. If the value is null return an empty string. If the model is present
 * on the audit then an attempt to use a plugin presenter will occur, otherwise the value is returned
 * withou any modification.
 *
 * @param array $audit      The audit data.
 * @param array $auditDelta The aduit delta array.
 * @param string $valueType Whether the value to display is old or new.
 * @return string The value to present.
 */
	public function presentValue($audit, $auditDelta, $valueType) {
		$value = $auditDelta[$valueType . '_value'];

		if ($value === null) {
			return '';
		}

		if (empty($audit['model'])) {
			return $value;
		}

		return self::_callPluginPresenter('presentValue', $audit, $auditDelta, $value);
	}

/**
 * Check if a value should appear in the audit log
 *
 * @param array $audit      The audit data.
 * @param array $auditDelta The aduit delta array.
 * @return bool Whether to present. Defaults to true unless overridden.
 */
	public function shouldPresentProperty($audit, $auditDelta) {
		return self::_callPluginPresenter('shouldPresentProperty', $audit, $auditDelta, true);
	}

/**
 * This method will attempt to call a plugin presenter. The plugin selected is based off of the model
 * that was used for the audit. The name of the plugin presenter should be called
 * `{pluginName}AuditLogPresenter` and should reside in the `Lib` folder. If a presenter is required to be
 * extended then a plugin presenter prefixed with the plugin name again will be looked for in the project
 * Lib folder.
 *
 * @param string $method     The method to call in the plugin presenter.
 * @param array  $audit      The audit data.
 * @param array  $auditDelta The aduit delta array.
 * @param string $value      The current value to present.
 * @return string The value to present.
 */
	protected function _callPluginPresenter($method, $audit, $auditDelta, $value) {
		list($plugin, $modelAlias) = pluginSplit($audit['model']);

		//Check for an extended plugin presenter first.
		$extendedAuditDeltaPresenter = $plugin . $plugin . 'AuditLogPresenter';

		App::uses($extendedAuditDeltaPresenter, 'Lib');

		if (class_exists($extendedAuditDeltaPresenter) && method_exists($extendedAuditDeltaPresenter, $method)) {
			return $extendedAuditDeltaPresenter::{$method}($audit, $auditDelta, $value);
		}

		//If an extended version couldn't be found check for a presenter
		$auditDeltaPresenter = $plugin . 'AuditLogPresenter';

		App::uses($auditDeltaPresenter, $plugin . '.Lib');

		if (!class_exists($auditDeltaPresenter) || !method_exists($auditDeltaPresenter, $method)) {
			return $value;
		}

		return $auditDeltaPresenter::{$method}($audit, $auditDelta, $value);
	}

/**
 * Presenter method to present a boolean value which can be used in plugin presenters.
 *
 * @param string $value The current value to present.
 * @return string The value to present.
 */
	protected function _presentBoolean($value) {
		if (empty($value)) {
			return 'No';
		} else {
			return 'Yes';
		}
	}

/**
 * Presenter method to replace an id field with the displayname of the field.
 * Initialises the model to find and then searches using the current value. Returns the displayField value
 * of the search. If no record is found then the value is returned without modification.
 *
 * @param string $modelClass The class of the model to initialise.
 * @param string $value      The current value to present.
 * @return string The value to present.
 */
	protected function _lookupValue($modelClass, $value) {
		$Model = EvClassRegistry::init($modelClass);
		$modelData = $Model->findById($value);

		if (!empty($modelData)) {
			$value = $modelData[$Model->alias][$Model->displayField];
		}

		return $value;
	}
}
