<?php

App::uses('AuditableBehavior', 'AuditLog.Model/Behavior');

App::uses('CakeSession', 'Model/Datasource');

class EvAuditLogAuditableBehavior extends AuditableBehavior {

/**
 * Get the current user from the auth component. Have to use the cake session statically as this is a
 * behavior. This isn't the proper way to access the auth session but is the only way we are able to
 * detect which user has made a change. If it isn't possible to acquire the user or a user didn't
 * make the change then it defaults to a system change.
 *
 * @param Model $Model Cake model the behavior is running on.
 * @return array Containing the id and description of the current user. Id is only set if a user is found.
 */
	public function currentUser(Model $Model) {
		$currentUser = [
			'id' => null,
			'description' => null,
		];

		$sessionUser = CakeSession::read('Auth.User');

		if (!empty($sessionUser)) {
			$currentUser['id'] = $sessionUser['User']['id'];
			$currentUser['description'] = $sessionUser['User']['email'];
		} else {
			$currentUser['description'] = 'System Change';
		}

		return $currentUser;
	}

/**
 * Get the audits for the current model. Can be limited to just create or edit events. Audits from associated
 * models can be contained by adding the models to the containLog behavior setting.
 *
 * @param Model       $Model Cake model to get audits of.
 * @param int         $id    The id of the model to find audits of.
 * @param null|string $event Empty to get all audits, create or edit to limit to those events.
 * @return array Found audits for the current model.
 */
	public function getAudits(Model $Model, $id, $event = null) {
		if ($this->_isAuditLogModel($Model) || empty($id)) {
			return null;
		}

		$Model->bindModel(
			[
				'hasMany' => [
					'AuditLog.Audit'
				]
			]
		);

		$containLogs = !empty($this->settings[$Model->alias]['containLog']) ? $this->settings[$Model->alias]['containLog'] : null;

		//Get the ids of the audits we want to get.
		$auditIds = $this->getAuditIds($Model, $id, $event, $containLogs);

		$audits = [];
		if (!empty($auditIds)) {
			//Find the actual audits to get with the deltas and order them.
			$audits = $Model->Audit->find(
				'all',
				[
					'conditions' => [
						'Audit.id' => $auditIds,
					],
					'order' => 'Audit.event DESC, Audit.created DESC',
					'group' => 'Audit.id',
					'contain' => [
						'AuditDelta' => [
							'order' => 'property_name ASC'
						]
					]
				]
			);
		}

		return $audits;
	}

/**
 * Get a list of audit ids for the current instance of the model. Limit to create or edit events and contain
 * associated model audits by providing a contain array. The list is used to get a full search for audits
 * that contains the deltas and is ordered.
 *
 * @param Model       $Model       Cake model of the current instance to get audit ids for.
 * @param int|array   $id          Id of the model to get audits for. If an array of ids is passed then all audits will be found.
 * @param null|string $event       Empty to get all audits, create or edit to limit to those events.
 * @param array       $containLogs The audit logs of associated models to contain in the audit list.
 * @return array List of audit ids that the current model instance has.
 */
	public function getAuditIds(Model $Model, $id, $event, $containLogs = []) {
		if ($this->_isAuditLogModel($Model) || empty($id)) {
			return null;
		}

		if (!isset($Model->hasMany['Audit'])) {
			$Model->bindModel(
				[
					'hasMany' => [
						'AuditLog.Audit',
					],
				]
			);
		}

		//Get the audit logs for the current model id.
		$auditConditions = [
			'Audit.model' => $Model->plugin ? $Model->plugin . '.' . $Model->alias : $Model->alias,
			'Audit.entity_id' => $id,
		];

		if (!empty($event)) {
			$auditConditions['Audit.event'] = strtoupper($event);
		}

		$auditIds = $Model->Audit->find(
			'all',
			[
				'fields' => [
					'Audit.id',
				],
				'conditions' => $auditConditions,
			]
		);

		$auditIds = Hash::extract($auditIds, '{n}.Audit.id');

		//Get any associated audit logs that are to be contained.
		$associatedModels = $Model->getAssociated();
		if (!empty($containLogs) && is_array($containLogs) && !empty($associatedModels)) {
			foreach ($containLogs as $containIndex => $contain) {
				$containModel = $contain;
				if (is_array($contain)) {
					$containModel = $containIndex;
				}

				if (!isset($containModel, $associatedModels)) {
					continue;
				}

				if (
					!$this->_isAuditLogModel($Model->{$containModel}) &&
					$Model->{$containModel}->hasMethod('getAuditIds')
				) {
					$associatedModel = $Model->{$containModel};

					//Get the ids of the assocaited data via the foreign key.
					$associatedForeignKey = $Model->{$associatedModels[$containModel]}[$containModel]['foreignKey'];
					$assocaitedIds = $associatedModel->find(
						'all',
						[
							'fields' => [
								$associatedModel->alias . '.id',
							],
							'conditions' => [
								$associatedModel->alias . '.' . $associatedForeignKey => $id,
							],
						]
					);

					$assocaitedIds = Hash::extract($assocaitedIds, '{n}.' . $associatedModel->alias . '.id');
					if (!empty($assocaitedIds)) {
						$assocaitedAuditIds = $associatedModel->getAuditIds($assocaitedIds, $event, $contain);

						if (!empty($assocaitedAuditIds)) {
							$auditIds = Hash::merge($auditIds, $assocaitedAuditIds);
						}
					}
				}
			}
		}

		return $auditIds;
	}
}
